// Generated by CoffeeScript 1.7.1
(function() {
  var DebugGraph, util,
    __slice = [].slice,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  util = require('util');

  DebugGraph = (function() {
    function DebugGraph() {
      this.nodes = {};
      this.edges = [];
      this.success = false;
      this.failure = false;
    }

    DebugGraph.prototype.addEdge = function(from, to) {
      this.edges.push({
        from: from,
        to: to
      });
      if (this.nodes[from] != null) {
        this.nodes[from].children.push(to);
      }
      if (this.nodes[to] != null) {
        this.nodes[to].parents.push(from);
      }
      if (to === "success") {
        this.success = true;
      }
      if (to === "failure") {
        return this.failure = true;
      }
    };

    DebugGraph.prototype.addNode = function(name, parser, state) {
      var edge, node, _i, _len, _ref;
      node = {
        parser: parser,
        state: state,
        parents: [],
        children: []
      };
      _ref = this.edges;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        edge = _ref[_i];
        if (edge.from === name) {
          node.children.push(edge.to);
        }
        if (edge.to === name) {
          node.parents.push(edge.from);
        }
      }
      return this.nodes[name] = node;
    };

    DebugGraph.prototype.filterOut = function() {
      var c, edgesToRemove, k, kinds, name, node, nodesToRemove, p, v, _i, _j, _k, _l, _len, _len1, _len2, _len3, _len4, _m, _ref, _ref1, _ref2, _ref3, _ref4, _ref5, _ref6, _results;
      kinds = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      nodesToRemove = [];
      _ref = this.nodes;
      for (k in _ref) {
        v = _ref[k];
        if (_ref1 = (_ref2 = v.parser) != null ? _ref2.kind : void 0, __indexOf.call(kinds, _ref1) >= 0) {
          nodesToRemove.push(k);
        }
      }
      _results = [];
      for (_i = 0, _len = nodesToRemove.length; _i < _len; _i++) {
        name = nodesToRemove[_i];
        edgesToRemove = [];
        node = this.nodes[name];
        if (node.parents.length > 1 && node.children.length > 1) {
          continue;
        }
        _ref3 = node.parents;
        for (_j = 0, _len1 = _ref3.length; _j < _len1; _j++) {
          p = _ref3[_j];
          edgesToRemove.push({
            from: p,
            to: name
          });
          if (this.nodes[p] != null) {
            this.nodes[p].children = this.nodes[p].children.filter(function(c) {
              return c !== name;
            });
          }
        }
        _ref4 = node.children;
        for (_k = 0, _len2 = _ref4.length; _k < _len2; _k++) {
          c = _ref4[_k];
          edgesToRemove.push({
            from: name,
            to: c
          });
          if (this.nodes[c] != null) {
            this.nodes[c].parents = this.nodes[c].parents.filter(function(p) {
              return p !== name;
            });
          }
        }
        _ref5 = node.parents;
        for (_l = 0, _len3 = _ref5.length; _l < _len3; _l++) {
          p = _ref5[_l];
          _ref6 = node.children;
          for (_m = 0, _len4 = _ref6.length; _m < _len4; _m++) {
            c = _ref6[_m];
            this.addEdge(p, c);
          }
        }
        delete this.nodes[name];
        _results.push(this.edges = this.edges.filter(function(edge) {
          var e, keep, _len5, _n;
          keep = true;
          for (_n = 0, _len5 = edgesToRemove.length; _n < _len5; _n++) {
            e = edgesToRemove[_n];
            if (e.from === edge.from && e.to === edge.to) {
              keep = false;
            }
          }
          return keep;
        }));
      }
      return _results;
    };

    DebugGraph.prototype.toDot = function(maxLength) {
      var data, description, edge, edges, k, label, nodes, v;
      if (maxLength == null) {
        maxLength = 40;
      }
      this.filterOut("chain", "seq", "onMatch", "matchIf");
      edges = (function() {
        var _i, _len, _ref, _results;
        _ref = this.edges;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          edge = _ref[_i];
          _results.push("  \"" + edge.from + "\" -> \"" + edge.to + "\";");
        }
        return _results;
      }).call(this);
      nodes = (function() {
        var _ref, _results;
        _ref = this.nodes;
        _results = [];
        for (k in _ref) {
          v = _ref[k];
          if (!(v.parser != null)) {
            continue;
          }
          description = v.parser.description();
          if (description.length > maxLength) {
            description = v.parser.kind + "...";
          }
          description = description.replace("\\", "\\\\").replace("\"", "\\\"");
          label = "@" + (v.state.pos()) + " " + v.state.depth + ": " + description + "\\n'" + (v.state.around(4)) + "'";
          label = label.replace(/"/g, "\\\"");
          _results.push("  \"" + k + "\" [label=\"" + label + "\"];");
        }
        return _results;
      }).call(this);
      data = ["digraph packrattle {", "  node [fontname=Courier];"];
      data = data.concat(edges);
      data.push("");
      data = data.concat(nodes);
      data.push("");
      data.push("  \"start\" [shape=rect, style=filled, fillcolor=yellow];");
      if (this.success) {
        data.push("  \"success\" [shape=rect, style=filled, fillcolor=green];");
      }
      if (this.failure) {
        data.push("  \"failure\" [shape=rect, style=filled, fillcolor=red];");
      }
      data.push("}");
      return data.join("\n") + "\n";
    };

    return DebugGraph;

  })();

  exports.DebugGraph = DebugGraph;

}).call(this);
