// Generated by CoffeeScript 1.7.1
(function() {
  var Location, Match, NoMatch, ParserState, Trampoline, debug_graph, util;

  util = require('util');

  debug_graph = require("./debug_graph");

  Trampoline = require("./trampoline").Trampoline;

  Location = (function() {
    function Location(pos, xpos, lineno) {
      this.pos = pos;
      this.xpos = xpos;
      this.lineno = lineno;
    }

    Location.prototype.toString = function() {
      return "(pos=" + this.pos + ", xpos=" + this.xpos + ", lineno=" + this.lineno + ")";
    };

    Location.prototype.advance = function(n, text) {
      var lineno, pos, xpos;
      pos = this.pos;
      xpos = this.xpos;
      lineno = this.lineno;
      while (pos < this.pos + n) {
        if (text[pos++] === '\n') {
          lineno++;
          xpos = 0;
        } else {
          xpos++;
        }
      }
      return new Location(pos, xpos, lineno);
    };

    return Location;

  })();

  ParserState = (function() {
    function ParserState(text, pos, end) {
      if (pos == null) {
        pos = 0;
      }
      if (end == null) {
        end = text.length;
      }
      this.loc = new Location(pos, 0, 0);
      this.depth = 0;
      this.stateName = null;
      this.internal = {
        text: text,
        end: end,
        trampoline: new Trampoline(this),
        "debugger": null
      };
    }

    ParserState.prototype.clone = function() {
      var rv;
      rv = Object.create(this.__proto__);
      rv.loc = this.loc;
      rv.oldloc = this.oldloc;
      rv.endloc = this.endloc;
      rv.depth = this.depth;
      rv.stateName = this.stateName;
      rv.internal = this.internal;
      return rv;
    };

    ParserState.prototype.toString = function() {
      var truncated;
      truncated = this.internal.text.length > 10 ? "'" + this.internal.text.slice(0, 10) + "...'" : "'" + this.internal.text + "'";
      return "ParserState(text=" + truncated + ", loc=" + this.loc + ", depth=" + this.depth + ")";
    };

    ParserState.prototype.startDebugGraph = function() {
      return this.internal["debugger"] = {
        graph: new debug_graph.DebugGraph()
      };
    };

    ParserState.prototype.advance = function(n) {
      var rv;
      rv = this.clone();
      rv.oldloc = this.loc;
      rv.loc = this.loc.advance(n, this.internal.text);
      return rv;
    };

    ParserState.prototype.flip = function() {
      var loc, rv;
      loc = this.oldloc != null ? this.oldloc : this.loc;
      rv = this.clone();
      rv.loc = loc;
      rv.endloc = this.loc;
      return rv;
    };

    ParserState.prototype.backfill = function(otherState) {
      var rv;
      rv = this.clone();
      if (this.endloc != null) {
        rv.loc = otherState.loc;
      } else {
        rv.oldloc = otherState.oldloc != null ? otherState.oldloc : otherState.loc;
      }
      return rv;
    };

    ParserState.prototype.pos = function() {
      return this.loc.pos;
    };

    ParserState.prototype.xpos = function() {
      return this.loc.xpos;
    };

    ParserState.prototype.endpos = function() {
      return (this.endloc || this.loc).pos;
    };

    ParserState.prototype.lineno = function() {
      return this.loc.lineno;
    };

    ParserState.prototype.line = function(pos) {
      var end, lend, lstart, text;
      if (pos == null) {
        pos = this.loc.pos;
      }
      text = this.internal.text;
      end = this.internal.end;
      lstart = pos;
      lend = pos;
      if (lstart > 0 && text[lstart] === '\n') {
        lstart--;
      }
      while (lstart > 0 && text[lstart] !== '\n') {
        lstart--;
      }
      if (text[lstart] === '\n') {
        lstart++;
      }
      while (lend < end && text[lend] !== '\n') {
        lend++;
      }
      return text.slice(lstart, lend);
    };

    ParserState.prototype.around = function(width) {
      var left, line, right, xpos;
      line = this.line();
      xpos = this.xpos();
      left = xpos - width;
      right = xpos + width;
      if (left < 0) {
        left = 0;
      }
      if (right >= line.length) {
        right = line.length - 1;
      }
      return line.slice(left, xpos) + "[" + (line[xpos] || "") + "]" + line.slice(xpos + 1, right + 1);
    };

    ParserState.prototype.getCache = function(parser) {
      return this.internal.trampoline.getCache(parser, this);
    };

    ParserState.prototype.addJob = function(description, job) {
      return this.internal.trampoline.push(this.depth, description, job);
    };

    ParserState.prototype.deeper = function(parser) {
      var newStateName, rv, _ref;
      rv = this.clone();
      rv.depth = this.depth + 1;
      if (((_ref = this.internal["debugger"]) != null ? _ref.graph : void 0) == null) {
        return rv;
      }
      newStateName = "" + parser.id + ":" + this.loc.pos;
      rv.stateName = newStateName;
      this.internal["debugger"].graph.addNode(newStateName, parser, rv);
      if (this.stateName != null) {
        this.internal["debugger"].graph.addEdge(this.stateName, newStateName);
      }
      return rv;
    };

    ParserState.prototype.logSuccess = function() {
      var _ref;
      if (((_ref = this.internal["debugger"]) != null ? _ref.graph : void 0) != null) {
        return this.internal["debugger"].graph.addEdge(this.stateName, "success");
      }
    };

    ParserState.prototype.logFailure = function() {
      var _ref;
      if (((_ref = this.internal["debugger"]) != null ? _ref.graph : void 0) != null) {
        return this.internal["debugger"].graph.addEdge(this.stateName, "failure");
      }
    };

    ParserState.prototype.debugGraphToDot = function() {
      var _ref;
      if (((_ref = this.internal["debugger"]) != null ? _ref.graph : void 0) == null) {
        return;
      }
      return this.internal["debugger"].graph.toDot();
    };

    ParserState.prototype.toSquiggles = function() {
      var endloc, endxpos, line, _i, _j, _ref, _ref1, _results, _results1;
      line = this.line();
      endloc = this.endloc || this.loc;
      endxpos = endloc.xpos;
      if (endloc.lineno !== this.loc.lineno) {
        endxpos = line.length;
      }
      if (endxpos === this.loc.xpos) {
        endxpos += 1;
      }
      return [
        line, (function() {
          _results = [];
          for (var _i = 0, _ref = this.loc.xpos; 0 <= _ref ? _i < _ref : _i > _ref; 0 <= _ref ? _i++ : _i--){ _results.push(_i); }
          return _results;
        }).apply(this).map(function() {
          return " ";
        }).join("") + (function() {
          _results1 = [];
          for (var _j = _ref1 = this.loc.xpos; _ref1 <= endxpos ? _j < endxpos : _j > endxpos; _ref1 <= endxpos ? _j++ : _j--){ _results1.push(_j); }
          return _results1;
        }).apply(this).map(function() {
          return "~";
        }).join("")
      ];
    };

    return ParserState;

  })();

  Match = (function() {
    function Match(state, match, commit) {
      this.state = state;
      this.match = match;
      this.commit = commit != null ? commit : false;
      this.ok = true;
    }

    Match.prototype.toString = function() {
      return "Match(state=" + this.state + ", match=" + (util.inspect(this.match)) + ", commit=" + this.commit + ")";
    };

    Match.prototype.equals = function(other) {
      return other.ok && this.match === other.match && this.state.pos() === other.state.pos();
    };

    return Match;

  })();

  NoMatch = (function() {
    function NoMatch(state, message, abort) {
      this.state = state;
      this.message = message;
      this.abort = abort != null ? abort : false;
      this.ok = false;
    }

    NoMatch.prototype.toString = function() {
      return "NoMatch(state=" + this.state + ", message='" + this.message + "', abort=" + this.abort + ")";
    };

    NoMatch.prototype.equals = function(other) {
      return (!other.ok) && this.state.pos() === other.state.pos() && this.message === other.message;
    };

    return NoMatch;

  })();

  exports.ParserState = ParserState;

  exports.Match = Match;

  exports.NoMatch = NoMatch;

}).call(this);
